#import "Tweak.h"
#import "Preferences.h"
#import <LibStatusBar/LSStatusBarItem.h>
#import <notify.h>
#import <SpringBoard/SBApplication.h>
#import <SpringBoard/SBApplicationController.h>
#import <SpringBoard/SBApplicationIcon.h>
#import <SpringBoard/SBIcon.h>
#import <SpringBoard/SBAwayController.h>
#import <SpringBoard7/SBMediaController.h>
#import <SpringBoard7/SBUserAgent.h>
#import <SpringBoard7/SBSoundPreferences.h>
#import <SpringBoard7/SBTelephonyManager.h>
#import <SpringBoard7/BluetoothManager.h>
#import <SpringBoard7/SBStatusBarStateAggregator.h>
#import <SpringBoard7/SBOrientationLockManager.h>
#import <UIKit7/UIStatusBarItem.h>
#import <AVFoundation/AVAudioSession.h>


#pragma mark #region [ Private Variables ]
static ONPreferences* preferences;
static NSMutableDictionary* statusBarItems = [[NSMutableDictionary alloc] init];
static NSMutableDictionary* currentIconSetList = [[NSMutableDictionary alloc] init];
static NSMutableDictionary* trackedBadges = [[NSMutableDictionary alloc] init];
static NSMutableDictionary* trackedCount = [[NSMutableDictionary alloc] init];
static LSStatusBarItem* silentIconItem;
static LSStatusBarItem* vibrateIconItem;
static LSStatusBarItem* tetherIconItem;
//static LSStatusBarItem* airplaneIconItem;
static LSStatusBarItem* airPlayIconItem;
static LSStatusBarItem* alarmIconItem;
static LSStatusBarItem* bluetoothIconItem;
static LSStatusBarItem* quietIconItem;
static LSStatusBarItem* rotationLockIconItem;
static LSStatusBarItem* vPNIconItem;
//static BOOL isAirplane = false;
static BOOL isAirPlay = false;
static BOOL isAlarm = false;
static BOOL isBluetooth = false;
static BOOL isQuiet = false;
static BOOL isRotation = false;
static BOOL isVPN = false;
static BOOL changeTether = false;
static id mailBadge = nil;
#pragma mark #endregion

#pragma mark #region [ Global Functions ]

//AirplaneMode:Airplane (Left)
//Battery (Right)
//BatteryPercent (Right)
//Bluetooth (Right)
//BluetoothBattery (Right)
//DataNetwork (Left)
//Home (Center)
//Indicator:AirPlay (Right)
//Indicator:Alarm (Right)
//Indicator:InCall (Right)
//Indicator:Navigation (Right)
//Indicator:Plus (Right)
//Indicator:Siri (Right)
//Indicator:TTY (Right)
//Indicator:VPN (Left/Right)
//NotCharging (Right)
//QuietMode:QuietMode (Right)
//ReturnToNavigation (Right)
//Service (Left)
//SignalStrength (Left)
//ThermalColor (Left/Right)
//Time (Center)
//Time (Right)


static inline NSString *IconNameFromItem(UIStatusBarItem *item)
{
	NSRange range = [[item description] rangeOfString:@"[" options:NSLiteralSearch];
	NSRange iconNameRange;
	iconNameRange.location = range.location + 1;
	iconNameRange.length = ((NSString *)[item description]).length - range.location - 2;
	return [[item description] substringWithRange:iconNameRange];
}

static void SetSystemIcon(NSString *name, bool enable)
{
	for (int i = 0; i < 100; i++)
	{
		UIStatusBarItem *item = [%c(UIStatusBarItem) itemWithType:i idiom:0];
		if (!item)
		{
			break;
		}

		if ([IconNameFromItem(item) isEqualToString:name])
		{
			[[%c(SBStatusBarStateAggregator) sharedInstance] _setItem:i enabled:enable];
			break;
		}
	}
}

static LSStatusBarItem* CreateStatusBarItem(NSString* uniqueName, NSString* iconName, bool onLeft)
{
	LSStatusBarItem* item = [[[%c(LSStatusBarItem) alloc] 
		initWithIdentifier:[NSString stringWithFormat:@"opennotifier.%@", uniqueName] 
		alignment:onLeft ? StatusBarAlignmentLeft : StatusBarAlignmentRight] autorelease];
				
	item.imageName = [NSString stringWithFormat:@"ON_%@", iconName];		
	return item;
}

static void ProcessApplicationIcon(NSString* identifier)
{
	if (!preferences.enabled) return;	
	
	ONApplication* app;
	if (!(app = [preferences getApplication:identifier])) return;
	bool shouldShow = [[trackedBadges objectForKey:identifier] boolValue];
	int count = [[trackedCount objectForKey:identifier] integerValue];
	bool isCountIcon = false;

	for (NSString* name in app.icons.allKeys)
	{
		if (![currentIconSetList.allKeys containsObject:name]) continue; // icon doesn't exist
		
		ONApplicationIcon* icon = [app.icons objectForKey:name];			
		bool onLeft;								
		switch (icon.alignment)
		{
			case ONIconAlignmentLeft: onLeft = true; break;
			case ONIconAlignmentRight: onLeft = false; break;
			default: onLeft = preferences.iconsOnLeft; break;
		}
				
		// avoid colliding with another icon with the same name	and alignment
		NSString* uniqueName = [NSString stringWithFormat:@"%@~%d", name, onLeft];

		if ([name hasPrefix:@"Count_"])
		{
			isCountIcon = true;
			if (count > 9)
			{
				name = [NSString stringWithFormat:@"Count%d%@", 10, [name substringFromIndex:5]];
			}
			else if (count > 0)
			{
				name = [NSString stringWithFormat:@"Count%d%@", count, [name substringFromIndex:5]];
			}
			else
			{
				name = [NSString stringWithFormat:@"Count%d%@", 1, name];
			}

			if (shouldShow)
			{
				// Remove icon so it can be readded with a different count value
				[statusBarItems removeObjectForKey:uniqueName];
			}
		}

		// applications may be sharing name and alignment so lets 
		// track it properly before we readd or remove it		
		NSMutableDictionary* uniqueIcon = [statusBarItems objectForKey:uniqueName];
		if (!uniqueIcon) uniqueIcon = [NSMutableDictionary dictionary];
		
		NSMutableArray* apps = [uniqueIcon objectForKey:ONApplicationsKey];
		if (!apps) apps = [NSMutableArray array];
		
		if (!shouldShow)
		{
			[apps removeObject:identifier];
			if (apps.count == 0) [statusBarItems removeObjectForKey:uniqueName];
		}
		else
		{	
			[apps addObject:identifier];
			[uniqueIcon setObject:apps forKey:ONApplicationsKey];
			
			if (![uniqueIcon.allKeys containsObject:ONIconNameKey] || isCountIcon)
				[uniqueIcon setObject:CreateStatusBarItem(uniqueName, name, onLeft) forKey:ONIconNameKey];
			
			[statusBarItems setObject:uniqueIcon forKey:uniqueName];			
		}
	}	
}

static void ReloadSettings()
{
	if (!preferences) preferences = ONPreferences.sharedInstance;
	else [preferences reload];
}

static bool isSilent()
{
	bool muted = false;
	if (%c(SBMediaController) && [%c(SBMediaController) instancesRespondToSelector:@selector(isRingerMuted)])
	{
		muted = [[%c(SBMediaController) sharedInstance] isRingerMuted];
	}
	else
	{
		// I'm not sure if this is needed or not but leaving it here just in case
		// it needs to be backwards compatible
		uint64_t state;
		int token;
		notify_register_check("com.apple.springboard.ringerstate", &token);
		notify_get_state(token, &state);
		notify_cancel(token);
		muted = (!state);
	}

	return muted;
}

static void UpdateVibrateIcon()
{
	if (vibrateIconItem)
	{
		[vibrateIconItem release];
		vibrateIconItem = nil;
	}

	if (preferences.vibrateModeEnabled && preferences.enabled)
	{
		bool vibrate = false;
		NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithContentsOfFile:@"/var/mobile/Library/Preferences/com.apple.springboard.plist"];
		if (dict)
		{
			if ([[dict valueForKey:@"ring-vibrate"] boolValue])
			{
				vibrate = true;
			}
			else if ([[dict valueForKey:@"silent-vibrate"] boolValue])
			{
				vibrate = isSilent();
			}
		}

		if (vibrate) vibrateIconItem = [CreateStatusBarItem(ONVibrateKey, ONVibrateKey, preferences.vibrateIconOnLeft) retain];
	}
}

static void VibrateModeSettingsChanged()
{
	ReloadSettings();
	UpdateVibrateIcon();
}

static void UpdateSilentIcon()
{
	if (silentIconItem) 
	{ 
		[silentIconItem release];
		silentIconItem = nil;
	}
		
	if (preferences.silentModeEnabled && preferences.enabled)
	{
		if (isSilent())
		{
			silentIconItem = [CreateStatusBarItem(ONSilentKey, ONSilentKey, preferences.silentIconOnLeft) retain];
		}
	}

	if (preferences.vibrateModeEnabled) UpdateVibrateIcon();
}

static void SilentModeSettingsChanged()
{
	ReloadSettings();	
	UpdateSilentIcon();
}

static void UpdateTetherIcon()
{
	if (tetherIconItem)
	{
		[tetherIconItem release];
		tetherIconItem = nil;
	}

	if (preferences.tetherModeEnabled && preferences.enabled)
	{
		SBTelephonyManager *tm = [%c(SBTelephonyManager) sharedTelephonyManager];
		if (tm)
		{
			int i = tm.numberOfNetworkTetheredDevices;
			BOOL isTethered = tm.isNetworkTethering;
			if (isTethered)
			{
				changeTether = true;
				[tm setIsNetworkTethering:NO withNumberOfDevices:0];
				[tm setIsNetworkTethering:YES withNumberOfDevices:i];
				changeTether = false;
				tetherIconItem = [CreateStatusBarItem(ONTetherKey, ONTetherKey, preferences.tetherIconOnLeft) retain];
			}
		}
	}
	else
	{
		SBStatusBarStateAggregator *sbsa = [%c(SBStatusBarStateAggregator) sharedInstance];
		if (sbsa) [sbsa _updateTetheringState];
	}
}

static void TetherModeSettingsChanged()
{
	ReloadSettings();
	UpdateTetherIcon();
}

//static void UpdateAirplaneIcon()
//{
//	if (airplaneIconItem)
//	{
//		[airplaneIconItem release];
//		airplaneIconItem = nil;
//	}
//
//	if (preferences.airplaneModeEnabled)
//	{
//		SBTelephonyManager *tm = [%c(SBTelephonyManager) sharedTelephonyManager];
//		if (tm && tm.isInAirplaneMode)
//		{
//			airplaneIconItem = [CreateStatusBarItem(ONAirplaneKey, ONAirplaneKey, preferences.airplaneIconOnLeft) retain];
//		}
//	}
//}

static void UpdateAirPlayIcon()
{
	if (airPlayIconItem)
	{
		[airPlayIconItem release];
		airPlayIconItem = nil;
	}

//	if (preferences.enabled && preferences.airPlayModeEnabled && isAirPlay > 0 && (preferences.airPlayAlwaysEnabled || isAirPlay > 1))
	if (preferences.airPlayModeEnabled && preferences.enabled && isAirPlay)
	{
		SetSystemIcon(@"Indicator:AirPlay (Right)", false);
		airPlayIconItem = [CreateStatusBarItem(ONAirPlayKey, ONAirPlayKey, preferences.airPlayIconOnLeft) retain];
	}
	else if ([[%c(SBMediaController) sharedInstance] isScreenSharing] || !isAirPlay)
	{
		SetSystemIcon(@"Indicator:AirPlay (Right)", isAirPlay);
	}
}

static void UpdateAlarmIcon()
{
	if (alarmIconItem)
	{
		[alarmIconItem release];
		alarmIconItem = nil;
	}

	if (preferences.alarmModeEnabled && isAlarm && preferences.enabled)
	{
		SetSystemIcon(@"Indicator:Alarm (Right)", false);
		alarmIconItem = [CreateStatusBarItem(ONAlarmKey, ONAlarmKey, preferences.alarmIconOnLeft) retain];
	}
	else
	{
		SetSystemIcon(@"Indicator:Alarm (Right)", isAlarm);
	}
}

static void UpdateBluetoothIcon()
{
	if (bluetoothIconItem)
	{
		[bluetoothIconItem release];
		bluetoothIconItem = nil;
	}

	BluetoothManager *bm = [%c(BluetoothManager) sharedInstance];
	isBluetooth = bm && bm.enabled;
	if (preferences.bluetoothModeEnabled && preferences.enabled)
	{
		SetSystemIcon(@"Bluetooth (Right)", false);
		if (bm && bm.enabled)
		{
			isBluetooth = true;
			if (bm.connected)
			{
				bluetoothIconItem = [CreateStatusBarItem(ONBluetoothConnectedKey, ONBluetoothConnectedKey, preferences.bluetoothIconOnLeft) retain];
			}
			else if (preferences.bluetoothAlwaysEnabled)
			{
				bluetoothIconItem = [CreateStatusBarItem(ONBluetoothKey, ONBluetoothKey, preferences.bluetoothIconOnLeft) retain];
			}
		}
	}
	else
	{
		SetSystemIcon(@"Bluetooth (Right)", isBluetooth);
	}
}

static void UpdateQuietIcon()
{
	if (quietIconItem)
	{
		[quietIconItem release];
		quietIconItem = nil;
	}

	if (preferences.quietModeEnabled && isQuiet && preferences.enabled)
	{
		SetSystemIcon(@"QuietMode:QuietMode (Right)", false);
		quietIconItem = [CreateStatusBarItem(ONQuietKey, ONQuietKey, preferences.quietIconOnLeft) retain];
	}
	else
	{
		SetSystemIcon(@"QuietMode:QuietMode (Right)", isQuiet);
	}
}

static void UpdateRotationLockIcon()
{
	if (rotationLockIconItem)
	{
		[rotationLockIconItem release];
		rotationLockIconItem = nil;
	}

	SBOrientationLockManager *bm = [%c(SBOrientationLockManager) sharedInstance];
	isRotation = bm && bm.isLocked;
	if (preferences.rotationLockModeEnabled && preferences.enabled)
	{
		SetSystemIcon(@"Indicator:RotationLock (Right)", false);
		if (bm && bm.isLocked)
		{
			isRotation = true;
			rotationLockIconItem = [CreateStatusBarItem(ONRotationLockKey, ONRotationLockKey, preferences.rotationLockIconOnLeft) retain];
		}
	}
	else
	{
		SetSystemIcon(@"Indicator:RotationLock (Right)", isRotation);
	}
}

static void UpdateVPNIcon()
{
	if (vPNIconItem)
	{
		[vPNIconItem release];
		vPNIconItem = nil;
	}

	SBTelephonyManager *tm = [%c(SBTelephonyManager) sharedTelephonyManager];
	isVPN = tm && tm.isUsingVPNConnection;
	if (preferences.vPNModeEnabled && preferences.enabled)
	{
		SetSystemIcon(@"Indicator:VPN (Left/Right)", false);
		if (tm && tm.isUsingVPNConnection)
		{
			isVPN = true;
			vPNIconItem = [CreateStatusBarItem(ONVPNKey, ONVPNKey, preferences.vPNIconOnLeft) retain];
		}
	}
	else
	{
		SetSystemIcon(@"Indicator:VPN (Left/Right)", isVPN);
	}

}

static BOOL isAirPlayActive()
{
	BOOL ret = NO;
	if ([[%c(SBMediaController) sharedInstance] isScreenSharing])
		return YES;
	if (!preferences.airPlayAlwaysEnabled)
		return NO;

	if ([%c(AVAudioSession) sharedInstance])
	{
		AVAudioSessionRouteDescription* route = [[%c(AVAudioSession) sharedInstance] currentRoute];
		for (AVAudioSessionPortDescription* desc in [route outputs])
		{
			if ([[desc portType] isEqualToString:@"AirPlay"]) //AVAudioSessionPortAirPlay
			{
				return YES;
			}
		}
	}

	return NO;
}

//static int isAirPlayActive()
//{
//	int ret = 0;
//	if ([[%c(SBMediaController) sharedInstance] isScreenSharing])
//		ret = 2;
//
//	if ([%c(AVAudioSession) sharedInstance])
//	{
//		AVAudioSessionRouteDescription* route = [[%c(AVAudioSession) sharedInstance] currentRoute];
//		for (AVAudioSessionPortDescription* desc in [route outputs])
//		{
//			if ([[desc portType] isEqualToString:@"AirPlay"]) //AVAudioSessionPortAirPlay
//			{
//				ret += 1;
//				break;
//			}
//		}
//	}
//
//	return ret;
//}

static void AirPlayModeSettingsChanged()
{
	ReloadSettings();
	isAirPlay = isAirPlayActive();
	UpdateAirPlayIcon();
}

static void AlarmModeSettingsChanged()
{
	ReloadSettings();
	UpdateAlarmIcon();
}

static void BluetoothModeSettingsChanged()
{
	ReloadSettings();
	UpdateBluetoothIcon();
}

static void QuietModeSettingsChanged()
{
	ReloadSettings();
	UpdateQuietIcon();
}

static void RotationLockModeSettingsChanged()
{
	ReloadSettings();
	UpdateRotationLockIcon();
}

static void VPNModeSettingsChanged()
{
	ReloadSettings();
	UpdateVPNIcon();
}

static void HideMailSettingsChanged()
{
	ReloadSettings();

	SBApplicationController* sbac = %c(SBApplicationController);
	if (sbac != NULL)
	{
		SBApplication *mailApp = [[sbac sharedInstance] applicationWithDisplayIdentifier:@"com.apple.mobilemail"];
		if (mailApp != NULL)
		{
			SBApplicationIcon *mailAppIcon = [[%c(SBApplicationIcon) alloc] initWithApplication:mailApp];
			if (mailAppIcon != NULL)
			{
				[mailAppIcon setBadge:mailBadge];
			}
		}
	}
}

static void IconSettingsChanged()
{
	ReloadSettings();
	
	[statusBarItems removeAllObjects];

	if (silentIconItem)
	{
		[silentIconItem release];
		silentIconItem = nil;
	}
	if (vibrateIconItem)
	{
		[vibrateIconItem release];
		vibrateIconItem = nil;
	}
	if (tetherIconItem)
	{
		[tetherIconItem release];
		tetherIconItem = nil;
	}
//	if (airplaneIconItem)
//	{
//		[airplaneIconItem release];
//		airplaneIconItem = nil;
//	}
	if (airPlayIconItem)
	{
		[airPlayIconItem release];
		airPlayIconItem = nil;
	}
	if (alarmIconItem)
	{
		[alarmIconItem release];
		alarmIconItem = nil;
	}
	if (bluetoothIconItem)
	{
		[bluetoothIconItem release];
		bluetoothIconItem = nil;
	}
	if (quietIconItem)
	{
		[quietIconItem release];
		quietIconItem = nil;
	}
	if (rotationLockIconItem)
	{
		[rotationLockIconItem release];
		rotationLockIconItem = nil;
	}
	if (vPNIconItem)
	{
		[vPNIconItem release];
		vPNIconItem = nil;
	}

	if (!preferences.enabled)
	{
		SetSystemIcon(@"Bluetooth (Right)", isBluetooth);
		SetSystemIcon(@"Indicator:Alarm (Right)", isAlarm);
		isAirPlay = isAirPlayActive();
		SetSystemIcon(@"Indicator:AirPlay (Right)", isAirPlay);
		SetSystemIcon(@"QuietMode:QuietMode (Right)", isQuiet);
		SetSystemIcon(@"Indicator:RotationLock (Right)", isRotation);
		SetSystemIcon(@"Indicator:VPN (Left/Right)", isVPN);
		return;
	}

	UpdateSilentIcon();
	UpdateVibrateIcon();
	UpdateTetherIcon();
//	UpdateAirplaneIcon();
	UpdateAirPlayIcon();
	UpdateAlarmIcon();
	UpdateBluetoothIcon();
	UpdateQuietIcon();
	UpdateRotationLockIcon();
	UpdateVPNIcon();
	
	[trackedBadges.allKeys enumerateObjectsUsingBlock: ^(id key, NSUInteger index, BOOL* stop){
		ProcessApplicationIcon(key);
	}];		
}

static void InitBadges()
{
	if ([%c(SBIconViewMap) respondsToSelector:@selector(homescreenMap)]) {
		for (NSString *identifier in [[[%c(SBIconViewMap) homescreenMap] iconModel] visibleIconIdentifiers]) {
			ONApplication* app;
			if (!(app = [preferences getApplication:identifier])) continue;
			SBIcon *icon = (SBIcon *)[[[%c(SBIconViewMap) homescreenMap] iconModel] applicationIconForDisplayIdentifier:identifier];
			if (icon && [icon badgeNumberOrString]) {
				[trackedBadges setObject:NSBool(YES) forKey:identifier];
				[trackedCount setObject:[icon badgeNumberOrString] forKey:identifier];
				ProcessApplicationIcon(identifier);
			}
		}
	}
}
#pragma mark #endregion

#pragma mark #region [ SpringBoard ]
%hook SpringBoard

%new - (void)AVAudioRouteChanged:(NSNotification*)notification
{
	if (!preferences.airPlayModeEnabled || !preferences.enabled)
		return;

	PostNotification((CFStringRef)AirPlayModeChangedNotification);
}

-(id)init
{	
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
	ReloadSettings();	
	NSMutableArray* imageNames = [NSMutableArray arrayWithArray:[[NSFileManager defaultManager] 
		contentsOfDirectoryAtPath:@"/System/Library/Frameworks/UIKit.framework/" error:nil]
	];

	NSRegularExpression* regex = [NSRegularExpression regularExpressionWithPattern:IconRegexPattern
		options:NSRegularExpressionCaseInsensitive error:nil];
	
	for (NSString* name in imageNames)
	{		
		NSTextCheckingResult* match = [regex firstMatchInString:name options:0 range:NSMakeRange(0, name.length)];
		if (!match) continue;
		name = [name substringWithRange:[match rangeAtIndex:1]];
		[currentIconSetList setObject:[NSMutableSet setWithCapacity:1] forKey:name];
	}
	
	[pool drain];
	return %orig;
}

-(void)applicationDidFinishLaunching:(id)application
{
	%orig;
	UpdateSilentIcon();	
	UpdateVibrateIcon();
	UpdateTetherIcon();
//	UpdateAirplaneIcon();
	UpdateBluetoothIcon();

	isAirPlay = isAirPlayActive();
	if (isAirPlay && preferences.enabled)
	{
		UpdateAirPlayIcon();
	}

	AddObserver((CFStringRef)IconSettingsChangedNotification, IconSettingsChanged);
	AddObserver((CFStringRef)SilentModeChangedNotification, SilentModeSettingsChanged);
	AddObserver((CFStringRef)VibrateModeChangedNotification, VibrateModeSettingsChanged);
	AddObserver((CFStringRef)TetherModeChangedNotification, TetherModeSettingsChanged);
	
//	AddObserver((CFStringRef)AirplaneModeChangedNotification, AirplaneModeSettingsChanged);
	AddObserver((CFStringRef)AirPlayModeChangedNotification, AirPlayModeSettingsChanged);
	AddObserver((CFStringRef)AlarmModeChangedNotification, AlarmModeSettingsChanged);
	AddObserver((CFStringRef)BluetoothModeChangedNotification, BluetoothModeSettingsChanged);
	AddObserver((CFStringRef)QuietModeChangedNotification, QuietModeSettingsChanged);
	AddObserver((CFStringRef)RotationLockModeChangedNotification, RotationLockModeSettingsChanged);
	AddObserver((CFStringRef)VPNModeChangedNotification, VPNModeSettingsChanged);
	
	AddObserver((CFStringRef)HideMailChangedNotification, HideMailSettingsChanged);

	CFNotificationCenterRef center = CFNotificationCenterGetDarwinNotifyCenter();
	CFNotificationCenterAddObserver(center, NULL, (CFNotificationCallback)&UpdateVibrateIcon, CFSTR("com.apple.springboard.ring-vibrate.changed"), NULL, CFNotificationSuspensionBehaviorCoalesce);
	CFNotificationCenterAddObserver(center, NULL, (CFNotificationCallback)&UpdateVibrateIcon, CFSTR("com.apple.springboard.silent-vibrate.changed"), NULL, CFNotificationSuspensionBehaviorCoalesce);

	[%c(AVAudioSession) sharedInstance];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(AVAudioRouteChanged:) name:@"AVAudioSessionRouteChangeNotification" object:nil];



	#ifdef DEBUGPREFS
	dispatch_queue_t queue = dispatch_get_main_queue();
	dispatch_async(queue, 
	^{
		SBAwayController* c = [%c(SBAwayController) sharedAwayController];
		[c attemptUnlock];
		[c unlockWithSound:false];
		[[%c(SBUserAgent) sharedUserAgent] openURL:[NSURL URLWithString:@"prefs:root=OpenNotifier"] allowUnlock:true animated:true];
		dispatch_release(queue);	
	});
	#endif

	InitBadges();
}
%end
#pragma mark #endregion

#pragma mark #region [ SBMediaController ]
%hook SBMediaController
-(void)setRingerMuted:(bool)change
{
	%orig;
	UpdateSilentIcon();
}
%end
#pragma mark #endregion

#pragma mark #region [ SBSoundPreferences ]
%hook SBSoundPreferences
-(void)userDefaultsDidChanged:(id)arg1
{
	%orig;
	if (preferences.vibrateModeEnabled)
	{
		UpdateVibrateIcon();
	}
}
%end
#pragma mark #endregion

#pragma mark #region [ SBTelephonyManager ]
%hook SBTelephonyManager
-(void)setIsNetworkTethering:(BOOL)arg1 withNumberOfDevices:(int)arg2
{
	%orig;
	if (!changeTether)
	{
		UpdateTetherIcon();
	}
}

//-(void)setIsInAirplaneMode:(BOOL)arg1
//{
//	isAirplane = arg1;
//	%orig;
//	UpdateAirplaneIcon();
//}
%end
#pragma mark #endregion

#pragma mark #region [ SBCCSettingsSectionController ]
%hook SBCCSettingsSectionController
-(void)_setDNDEnabled:(BOOL)arg1 updateServer:(BOOL)arg2
{
	isQuiet = arg1;
	%orig;
	if (preferences.quietModeEnabled)
	{
		UpdateQuietIcon();
	}
}
%end
#pragma mark #endregion

#pragma mark #region [ SBStatusBarStateAggregator ]
%hook SBStatusBarStateAggregator
-(void)_updateTetheringState
{
	if (preferences.tetherModeEnabled)
	{
		SBTelephonyManager *tm = [%c(SBTelephonyManager) sharedTelephonyManager];
		if (tm && tm.isNetworkTethering)
		{
			return;
		}
	}
	%orig;
}

//-(void)_updateAirplaneMode
//{
//	if (preferences.airplaneModeEnabled)
//	{
//		UpdateAirplaneIcon();
//		return;
//	}
//	%orig;
//}

-(void)_updateAirplayItem
{
	if (preferences.airPlayModeEnabled)
	{
		return;
	}
	%orig;
}

-(void)setAlarmEnabled:(BOOL)arg1
{
	isAlarm = arg1;
	%orig;
}

-(void)_updateAlarmItem
{
	if (preferences.alarmModeEnabled)
	{
		UpdateAlarmIcon();
		return;
	}
	%orig;
}

-(void)_updateBluetoothItem
{
	if (preferences.bluetoothModeEnabled)
	{
		UpdateBluetoothIcon();
		return;
	}
	%orig;
}

-(void)_updateQuietModeItem
{
	if (preferences.quietModeEnabled)
	{
		return;
	}
	%orig;
}

-(void)_updateRotationLockItem
{
	if (preferences.rotationLockModeEnabled)
	{
		UpdateRotationLockIcon();
		return;
	}
	%orig;
}

-(void)_updateVPNItem
{
	if (preferences.vPNModeEnabled)
	{
		UpdateVPNIcon();
		return;
	}
	%orig;
}
%end
#pragma mark #endregion

#pragma mark #region [ SBApplication ]
%hook SBApplication

-(void)setBadge:(id)badge
{
	id badgeCopy = badge;
	bool showBadge = !(badge == NULL || badge == nil || [badge isEqual:@""] || [badge isEqual:@"0"] || [badge isEqual:[NSNumber numberWithInt:0]]);
	if ([self.bundleIdentifier isEqualToString:@"com.apple.mobilemail"]) {
		mailBadge = badge;
		if (preferences.enabled && preferences.hideMail)
		{
			badge = nil;
		}
	}
	%orig(badge);

//	NSLog(@"SBApplication setBadge - identifier = %@ - %@, badge = %@", self.bundleIdentifier, self.displayIdentifier, badge);

	[trackedBadges setObject:NSBool(showBadge) forKey:self.bundleIdentifier];
	[trackedCount setObject:showBadge ? badgeCopy : @"0" forKey:self.bundleIdentifier];
	if (preferences.enabled) ProcessApplicationIcon(self.bundleIdentifier);
}
%end
#pragma mark #endregion
